@@ -22,9 +22,10 @@ import gc
 import genshi
 import sys
 import re
+import fnmatch
 import shutil
 
-from .config import LazygalConfig, LazygalWebgalConfig
+from .config import LazygalConfig
 from .config import USER_CONFIG_PATH, LazygalConfigDeprecated
 from .sourcetree import SOURCEDIR_CONFIGFILE
 from .pygexiv2 import GExiv2
@@ -33,6 +34,7 @@ from . import py2compat
 from . import make
 from . import pathutils
 from . import sourcetree
+from . import pindex
 from . import tpl
 from . import newsize
 from . import metadata
@@ -68,30 +70,35 @@ class SubgalSort(make.MakeTask):
     def build(self):
         logging.info(_("  SORTING pics and subdirs"))
 
-        if self.webgal_dir.subgal_sort_by[0] == 'exif':
+        order = self.webgal_dir.subgal_sort_by['order']
+        if order == 'exif':
             subgal_sortkey = lambda x: x.source_dir.latest_media_stamp()
-        elif self.webgal_dir.subgal_sort_by[0] == 'mtime':
+        elif order == 'mtime':
             subgal_sortkey = lambda x: x.source_dir.get_mtime()
-        elif self.webgal_dir.subgal_sort_by[0] == 'dirname'\
-                or self.webgal_dir.subgal_sort_by[0] == 'filename':  # Backward compatibility
+        elif order == 'numeric':
+            subgal_sortkey = lambda x: x.source_dir.name_numeric()
+        elif order == 'dirname' or order == 'filename':  # Backward compat
             subgal_sortkey = lambda x: x.source_dir.filename
         else:
             raise ValueError(_("Unknown sorting criterion '%s'")
                              % self.webgal_dir.subgal_sort_by[0])
         self.webgal_dir.subgals.sort(key=subgal_sortkey,
-                                     reverse=self.webgal_dir.subgal_sort_by[1])
+            reverse=self.webgal_dir.subgal_sort_by['reverse'])
 
-        if self.webgal_dir.pic_sort_by[0] == 'exif':
+        order = self.webgal_dir.pic_sort_by['order']
+        if order == 'exif':
             sortkey = lambda x: x.media.sortkey()
-        elif self.webgal_dir.pic_sort_by[0] == 'mtime':
+        elif order == 'mtime':
             sortkey = lambda x: x.media.get_mtime()
-        elif self.webgal_dir.pic_sort_by[0] == 'filename':
+        elif order == 'numeric':
+            sortkey = lambda x: x.media.name_numeric()
+        elif order == 'filename':
             sortkey = lambda x: x.media.filename
         else:
             raise ValueError(_("Unknown sorting criterion '%s'")
                              % self.webgal_dir.pic_sort_by[0])
         self.webgal_dir.medias.sort(key=sortkey,
-                                    reverse=self.webgal_dir.pic_sort_by[1])
+            reverse=self.webgal_dir.pic_sort_by['reverse'])
 
         # chain medias
         previous = None
@@ -152,14 +159,14 @@ class SubgalBreak(make.MakeTask):
         how_many_medias = 0
         subgals_it = iter([self.webgal_dir] + self.webgal_dir.get_all_subgals())
         for subgal in subgals_it:
-            how_many_medias += subgal.get_media_count()
+            how_many_medias += subgal.pindex.get_media_count()
             galleries.append((subgal, subgal.medias))
             if how_many_medias > self.webgal_dir.thumbs_per_page:
                 self.webgal_dir.add_index_page(subgals, galleries)
                 galleries = []
                 how_many_medias = 0
 
-        if len(galleries) > 0:
+        if galleries:
             self.webgal_dir.add_index_page(subgals, galleries)
 
     def __fill_real_pagination(self):
@@ -200,10 +207,7 @@ class WebalbumMediaTask(make.GroupTask):
         self.browse_pages = {}
 
         for size_name in self.webgal.browse_sizes:
-            if self.webgal.newsizers[size_name] == 'original':
-                self.resized[size_name] = self.get_original()
-            else:
-                self.resized[size_name] = self.get_resized(size_name)
+            self.resized[size_name] = self.get_resized(size_name)
             self.add_dependency(self.resized[size_name])
 
             if self.webgal.original and not self.webgal.orig_base:
@@ -300,7 +304,7 @@ class WebalbumVideoTask(WebalbumMediaTask):
         return self.webvideo
 
 
-class WebalbumDir(make.FileMakeObject):
+class WebalbumDir(make.GroupTask):
     """
     This is a built web gallery with its files, thumbs and reduced pics.
     """
@@ -310,12 +314,12 @@ class WebalbumDir(make.FileMakeObject):
         self.path = os.path.join(album_dest_dir, self.source_dir.strip_root())
         if self.path.endswith(os.sep): self.path = os.path.dirname(self.path)
 
-        super(WebalbumDir, self).__init__(self.path)
+        super(WebalbumDir, self).__init__()
 
         self.progress = progress
 
         self.add_dependency(self.source_dir)
-        self.subgals = [s for s in subgals if s.get_all_media_count() > 0]
+        self._init_subgals = subgals
         for srcdir in self.source_dir.subdirs:
             self.add_dependency(srcdir)
         self.album = album
@@ -323,21 +327,29 @@ class WebalbumDir(make.FileMakeObject):
 
         self.flattening_dir = None
 
-        self.config = LazygalWebgalConfig(self.album.config)
+        self.config = LazygalConfig()
+        self.config.load(self.album.config)
         self.__configure()
 
-        tagfilters = self.config.getlist('webgal', 'filter-by-tag')
+        self.pindex = pindex.PersistentIndex(self)
+        self.add_dependency(self.pindex)
+
+    def populate_deps(self):
+        super(WebalbumDir, self).populate_deps()
+
+        self.subgals = []
+        for s in self._init_subgals:
+            if s.has_media_below():
+                self.subgals.append(s)
 
         self.medias = []
         self.sort_task = SubgalSort(self)
         self.sort_task.add_dependency(self.source_dir)
         for media in self.source_dir.medias:
-            self.sort_task.add_dependency(media)
-
-            if len(tagfilters) > 0 and media.info() is not None:
+            if self.tagfilters and media.info() is not None:
                 # tag-filtering is requested
                 res = True
-                for tagf in tagfilters:
+                for tagf in self.tagfilters:
                     # concatenate the list of tags as a string of words,
                     # space-separated.  to ensure that we match the full
                     # keyword and not only a subpart of it, we also surround
@@ -351,8 +363,11 @@ class WebalbumDir(make.FileMakeObject):
                         res = False
                         break
                 if res is False:
+                    self.media_done()
                     continue
 
+            self.sort_task.add_dependency(media)
+
             if media.type == 'image':
                 media_task = WebalbumImageTask(self, media)
             elif media.type == 'video':
@@ -364,29 +379,23 @@ class WebalbumDir(make.FileMakeObject):
             self.add_dependency(media_task)
 
         # Create the directory if it does not exist
-        if not os.path.isdir(self.path) and (self.get_media_count() > 0):
+        if not os.path.isdir(self.path) and self.has_media_below():
             logging.info(_("  MKDIR %%WEBALBUMROOT%%/%s"),
                          self.source_dir.strip_root())
             logging.debug("(%s)", self.path)
-            os.makedirs(self.path, mode=0o755)
-            self.stamp_delete() # Directory did not exist, mark it as so
+            os.makedirs(self.path)
 
-        if self.config.getboolean('webgal', 'dirzip')\
-                and self.get_media_count() > 1:
+        if self.pindex.dirzip():
             self.dirzip = genfile.WebalbumArchive(self)
             self.add_dependency(self.dirzip)
-        else:
-            self.dirzip = None
+            self.pindex.add_dependency(self.dirzip)
 
         self.index_pages = []
 
-        if (self.get_all_media_count() > 0) and not self.should_be_flattened():
+        if self.has_media_below() and not self.should_be_flattened():
             self.break_task = SubgalBreak(self)
 
             if self.thumbs_per_page > 0:
-                # FIXME: If pagination is 'on', galleries need to be sorted
-                # before being broken on multiple pages, and thus this slows
-                # down a lot the checking of a directory's need to be built.
                 self.break_task.add_dependency(self.sort_task)
 
             # This task is special because it populates dependencies. This is
@@ -398,9 +407,8 @@ class WebalbumDir(make.FileMakeObject):
         else:
             self.break_task = None
 
-    def __parse_browse_sizes(self, sizes_string):
-        for single_def in sizes_string.split(','):
-            name, string_size = single_def.split('=')
+    def __parse_browse_sizes(self, sizes_defs):
+        for name, string_size in sizes_defs.items():
             name = py2compat.u(name, locale.getpreferredencoding())
             if name == '':
                 raise ValueError(_("Sizes is a comma-separated list of size names and specs:\n\t e.g. \"small=640x480,medium=1024x768\"."))
@@ -420,30 +428,15 @@ class WebalbumDir(make.FileMakeObject):
             except newsize.NewsizeStringParseError:
                 raise ValueError(_("'%s' for size '%s' does not describe a known size syntax.") % (py2compat.u(size_string, locale.getpreferredencoding()), size_name, ))
 
-    def __parse_sort(self, sort_string):
-        try:
-            sort_method, reverse = sort_string.split(':')
-        except ValueError:
-            sort_method = sort_string
-            reverse = False
-        if reverse == 'reverse':
-            return sort_method, True
-        else:
-            return sort_method, False
-
     def __load_tpl_vars(self):
         # Load tpl vars from config
         tpl_vars = {}
         if self.config.has_section('template-vars'):
             tpl_vars = {}
             for option in self.config.options('template-vars'):
-                try:
-                    value = self.config.getboolean('template-vars', option)
-                    tpl_vars[option] = value
-                except ValueError:
-                    value = self.config.get('template-vars', option)
-                    value = py2compat.u(value, locale.getpreferredencoding())
-                    tpl_vars[option] = genshi.core.Markup(value)
+                value = self.config.get('template-vars', option)
+                value = py2compat.u(value, locale.getpreferredencoding())
+                tpl_vars[option] = genshi.core.Markup(value)
 
         return tpl_vars
 
@@ -455,7 +448,8 @@ class WebalbumDir(make.FileMakeObject):
         logging.debug(_("  Trying loading gallery configs: %s"),
                       ', '.join(map(self.source_dir.strip_root,
                                     config_files)))
-        self.config.read(config_files)
+        for c in config_files:
+            self.config.load_any(c)
 
         self.browse_sizes = []
         self.newsizers = {}
@@ -471,22 +465,22 @@ class WebalbumDir(make.FileMakeObject):
             self.config.get('webgal', 'default-style'))
         self.tpl_vars.update({'styles': styles})
 
-        self.set_original(self.config.getboolean('webgal', 'original'),
-                          self.config.getstr('webgal', 'original-baseurl'),
-                          self.config.getboolean('webgal', 'original-symlink'))
+        self.set_original(self.config.get('webgal', 'original'),
+                          self.config.get('webgal', 'original-baseurl'),
+                          self.config.get('webgal', 'original-symlink'))
 
-        self.thumbs_per_page = self.config.getint('webgal', 'thumbs-per-page')
+        self.thumbs_per_page = self.config.get('webgal', 'thumbs-per-page')
 
-        self.quality = self.config.getint('webgal', 'jpeg-quality')
+        self.quality = self.config.get('webgal', 'jpeg-quality')
         self.save_options = {}
-        if self.config.getboolean('webgal', 'jpeg-optimize'):
+        if self.config.get('webgal', 'jpeg-optimize'):
             self.save_options['optimize'] = True
-        if self.config.getboolean('webgal', 'jpeg-progressive'):
+        if self.config.get('webgal', 'jpeg-progressive'):
             self.save_options['progressive'] = True
 
-        self.pic_sort_by = self.__parse_sort(self.config.get('webgal', 'sort-medias'))
-        self.subgal_sort_by = self.__parse_sort(self.config.get('webgal', 'sort-subgals'))
-        self.filter_by_tag = self.config.get('webgal', 'filter-by-tag')
+        self.pic_sort_by = self.config.get('webgal', 'sort-medias')
+        self.subgal_sort_by = self.config.get('webgal', 'sort-subgals')
+        self.tagfilters = self.config.get('webgal', 'filter-by-tag')
 
         self.webalbumpic_bg = self.config.get('webgal', 'webalbumpic-bg')
         self.webalbumpic_type = self.config.get('webgal', 'webalbumpic-type')
@@ -497,7 +491,7 @@ class WebalbumDir(make.FileMakeObject):
         except ValueError:
             logging.error(_('Bad syntax for webalbumpic-size.'))
             sys.exit(1)
-        self.keep_gps = self.config.getboolean('webgal', 'keep-gps')
+        self.keep_gps = self.config.get('webgal', 'keep-gps')
 
     def set_original(self, original=False, orig_base=None, orig_symlink=False):
         self.original = original or orig_symlink
@@ -562,27 +556,19 @@ class WebalbumDir(make.FileMakeObject):
             all_subgals.extend(subgal.get_all_subgals())
         return all_subgals
 
-    def get_media_count(self, media_type=None):
-        if media_type is None:
-            return len(self.medias)
+    def has_media(self):
+        if self.tagfilters:
+            return True if self.medias else False
         else:
-            typed_media_count = 0
-            for mediatask in self.medias:
-                if mediatask.media.type == media_type:
-                    typed_media_count += 1
-            return typed_media_count
+            return True if self.source_dir.medias else False
 
-    def get_all_media_count(self):
-        count = len(self.medias)
-        for subgal in self.subgals:
-            count += subgal.get_all_media_count()
-        return count
-
-    def get_all_medias_tasks(self):
-        all_medias = list(self.medias)  # We want a copy here.
-        for subgal in self.subgals:
-            all_medias.extend(subgal.get_all_medias_tasks())
-        return all_medias
+    def has_media_below(self):
+        if self.has_media():
+            return True
+        for subgal in self._init_subgals:
+            if subgal.has_media_below():
+                return True
+        return False
 
     def should_be_flattened(self, path=None):
         if path is None: path = self.source_dir.path
@@ -592,7 +578,7 @@ class WebalbumDir(make.FileMakeObject):
     def flatten_below(self):
         if self.album.dir_flattening_depth is False:
             return False
-        elif len(self.source_dir.subdirs) > 0:
+        elif self.source_dir.subdirs:
             # As all subdirs are at the same level, if one should be flattened,
             # all should.
             return self.subgals[0].should_be_flattened()
@@ -629,6 +615,8 @@ class WebalbumDir(make.FileMakeObject):
             return ''
 
     def list_foreign_files(self):
+        self.call_populate_deps()
+
         if not os.path.isdir(self.path):
             return []
 
@@ -641,7 +629,8 @@ class WebalbumDir(make.FileMakeObject):
                                             DEST_SHARED_DIRECTORY_NAME))
 
         dirnames = [d.source_dir.name for d in self.subgals]
-        expected_dirs = map(lambda dn: os.path.join(self.path, dn), dirnames)
+        expected_dirs = list(map(lambda dn: os.path.join(self.path, dn),
+                             dirnames))
         for dest_file in os.listdir(self.path):
             dest_file = os.path.join(self.path, dest_file)
             # FIXME: No clue why this happens, but it happens!
@@ -653,22 +642,27 @@ class WebalbumDir(make.FileMakeObject):
 
         return foreign_files
 
+    def skip_media(self, src_media):
+        return False
+
+    def needs_build_quick(self):
+        """
+        Faster check if needs to be built.
+        """
+        if self._deps_populated or self.tagfilters:
+            return self.needs_build()
+
+        return self.pindex.needs_build()
+
     def build(self):
         for dest_file in self.list_foreign_files():
             self.album.cleanup(dest_file, self.path)
 
     def make(self, force=False):
-        needed_build = self.needs_build()
-
-        super(WebalbumDir, self).make(force or needed_build)
-
-        # Although we should have modified the directory contents and thus its
-        # mtime, it is possible that the directory mtime has not been updated
-        # if we regenerated without adding/removing pictures (to take into
-        # account a rotation for example). This is why we force directory mtime
-        # update here if something has been built.
-        if needed_build and os.path.isdir(self.path):
-            os.utime(self.path, None)
+        super(WebalbumDir, self).make(force)
+        if self.has_media_below(): # otherwise the dir is not created
+            os.utime(self.pindex._path, None) # update stamp mtime
+        self.update_build_status()
 
     def media_done(self):
         if self.progress is not None:
@@ -685,7 +679,7 @@ class SharedFiles(make.FileMakeObject):
         if not os.path.isdir(self.path):
             logging.info(_("MKDIR %SHAREDDIR%"))
             logging.debug("(%s)", self.path)
-            os.makedirs(self.path, mode=0o755)
+            os.makedirs(self.path)
 
         super(SharedFiles, self).__init__(self.path)
 
@@ -742,17 +736,24 @@ class AlbumGenProgress(object):
         self.updated()
 
     def __str__(self):
-        msg = _("Progress: dir %d/%d (%d%%), media %d/%d (%d%%)")\
-              % (self._dirs_done, self._dirs_total,
-                 100 * self._dirs_done // self._dirs_total,
-                 self._medias_done, self._medias_total,
-                 100 * self._medias_done // self._medias_total,
-                )
+        msg = []
+
+        if self._dirs_total > 0:
+            msg.append("dir %d/%d (%d%%)" \
+                  % (self._dirs_done, self._dirs_total,
+                     100 * self._dirs_done // self._dirs_total,
+                    ))
+
+        if self._medias_total > 0:
+            msg.append("media %d/%d (%d%%)" \
+                       % (self._medias_done, self._medias_total,
+                          100 * self._medias_done // self._medias_total,
+                         ))
 
         if self._task_percent is not None:
-            msg = msg + _(", current task %d%%") % self._task_percent
+            msg.append(_("current task %d%%") % self._task_percent)
 
-        return msg
+        return _("Progress: %s") % ', '.join(msg)
 
     def updated(self):
         pass
@@ -763,37 +764,38 @@ class Album(object):
     def __init__(self, source_dir, config=None):
         self.source_dir = os.path.abspath(source_dir)
 
-        if not mediautils.HAVE_GST:
-            logging.warning(_('Video support is disabled: could not load GStreamer'))
+        self.config = LazygalConfig(load_defaults=True)
 
-        self.config = LazygalConfig()
-
-        logging.info(_("Trying loading user config %s"), USER_CONFIG_PATH)
-        self.config.read(USER_CONFIG_PATH)
+        logging.debug(_("Trying loading user config %s"), USER_CONFIG_PATH)
+        self.config.load_any(USER_CONFIG_PATH)
 
         sourcedir_configfile = os.path.join(source_dir, SOURCEDIR_CONFIGFILE)
         if os.path.isfile(sourcedir_configfile):
-            logging.info(_("Loading root config %s"), sourcedir_configfile)
+            logging.debug(_("Loading root config %s"), sourcedir_configfile)
             try:
-                self.config.read(sourcedir_configfile)
+                self.config.load_any(sourcedir_configfile)
             except LazygalConfigDeprecated:
                 logging.error(_("'%s' uses a deprecated syntax: please refer to lazygal.conf(5) manual page."), sourcedir_configfile)
                 sys.exit(1)
         if config is not None:  # Supplied config
             self.config.load(config)
 
-        if self.config.getboolean('runtime', 'quiet'):
+        if self.config.get('runtime', 'quiet'):
             logging.getLogger().setLevel(logging.ERROR)
-        if self.config.getboolean('runtime', 'debug'):
+        if self.config.get('runtime', 'debug'):
             logging.getLogger().setLevel(logging.DEBUG)
             GExiv2.log_set_level(GExiv2.LogLevel.INFO)
 
-        self.clean_dest = self.config.getboolean('global', 'clean-destination')
-        self.force_gen_pages = self.config.getboolean('global', 'force-gen-pages')
+        self.clean_dest = self.config.get('global', 'clean-destination')
+        self.preserves = (self.config.get('global', 'preserve') +
+			 self.config.get('global', 'preserve_args'))
+        self.force_gen_pages = self.config.get('global', 'force-gen-pages')
 
         self.set_theme(self.config.get('global', 'theme'))
+        self.excludes = (self.config.get('global', 'exclude') +
+                        self.config.get('global', 'exclude_args'))
 
-        self.dir_flattening_depth = self.config.getint('global', 'dir-flattening-depth')
+        self.dir_flattening_depth = self.config.get('global', 'dir-flattening-depth')
 
         self.__statistics = None
 
@@ -811,6 +813,12 @@ class Album(object):
         if self.clean_dest:
             # Do not delete something out of dest-dir.
             assert pathutils.is_subdir_of(context_path, file_path)
+            tail = os.path.basename(file_path)
+            for pattern in self.preserves:
+                # Do not delete something the user wants to keep.
+                if fnmatch.fnmatch(tail, pattern):
+                    logging.info('  PRESERVE %s', file_path)
+                    return
             if os.path.isdir(file_path):
                 shutil.rmtree(file_path)
             else:
@@ -840,7 +848,7 @@ class Album(object):
             }
             for root, dirnames, filenames in pathutils.walk(self.source_dir):
                 dir_medias = len([f for f in filenames\
-                                  if sourcetree.MediaHandler.is_known_media(f)])
+                                  if sourcetree.MediaHandler.is_known_media(f, self)])
                 self.__statistics['total'] = self.__statistics['total']\
                                              + dir_medias
                 self.__statistics['bydir'][root] = dir_medias
@@ -848,13 +856,13 @@ class Album(object):
 
     def generate(self, dest_dir=None, progress=None):
         if dest_dir is None:
-            dest_dir = self.config.getstr('global', 'output-directory')
+            dest_dir = self.config.get('global', 'output-directory')
         else:
             dest_dir = py2compat.u(dest_dir, sys.getfilesystemencoding())
         sane_dest_dir = os.path.abspath(os.path.expanduser(dest_dir))
 
-        pub_url = self.config.getstr('global', 'puburl')
-        check_all_dirs = self.config.getboolean('runtime', 'check-all-dirs')
+        pub_url = self.config.get('global', 'puburl')
+        check_all_dirs = self.config.get('runtime', 'check-all-dirs')
 
         if self.is_in_sourcetree(sane_dest_dir):
             raise ValueError(_("Fatal error, web gallery directory is within source tree."))
@@ -917,13 +925,11 @@ class Album(object):
                 feed.push_dir(destgal)
                 destgal.register_feed(feed)
 
-            if check_all_dirs:
+            if check_all_dirs or destgal.needs_build_quick():
                 destgal.make()
-            elif destgal.needs_build():
-                destgal.make(force=True)  # avoid another needs_build() call in make()
             else:
                 if progress is not None:
-                    progress.media_done(len(destgal.medias))
+                    progress.media_done(self.stats()['bydir'][destgal.source_dir.path])
                 logging.info(_("  SKIPPED because of mtime, touch source or use --check-all-dirs to override."))
 
             # Force some memory cleanups, this is usefull for big albums.
