@@ -77,9 +77,9 @@ class ResizedImage(genfile.WebalbumFile):
         try:
             self.build()
         except IOError:
+            logging.error(_("  %s is BROKEN, skipped"),
+                          self.source_media.filename)
             if not self.source_media.broken:
-                logging.error(_("  %s is BROKEN, skipped"),
-                              self.source_media.filename)
                 self.source_media.broken = True
 
             # Make the system believe the file was built a long time ago.
@@ -90,7 +90,7 @@ class ResizedImage(genfile.WebalbumFile):
     def resize(self, im):
         self.source_media.get_size()  # Probe brokenness
         if self.source_media.broken:
-            raise IOError()
+            raise IOError('Failed to get media size')
 
         new_size = self.get_size()
 
@@ -100,15 +100,16 @@ class ResizedImage(genfile.WebalbumFile):
     def save(self, im):
         calibrated = False
         while not calibrated:
-            try:
-                im.save(self.path, quality=self.webgal.quality,
-                        **self.webgal.save_options)
-            except IOError as e:
-                if str(e).startswith('encoder error'):
-                    PILImageFile.MAXBLOCK = 2 * PILImageFile.MAXBLOCK
-                    continue
-                else:
-                    raise
+            with open(self.path, 'w+') as im_fp:
+                try:
+                    im.save(im_fp, 'jpeg', quality=self.webgal.quality,
+                            **self.webgal.save_options)
+                except IOError as e:
+                    if str(e).startswith('encoder error'):
+                        PILImageFile.MAXBLOCK = 2 * PILImageFile.MAXBLOCK
+                        continue
+                    else:
+                        raise
             calibrated = True
 
 
@@ -147,7 +148,10 @@ class ImageOtherSize(ResizedImage):
         return self.size
 
     def get_image(self):
-        return PILImage.open(self.source_media.path)
+        with open(self.source_media.path, 'rb') as im_fp:
+            im = PILImage.open(im_fp)
+            im.load()
+            return im
 
     TRANSPOSE_METHODS = {
         90 : PILImage.ROTATE_90,
@@ -183,7 +187,10 @@ class ImageOtherSize(ResizedImage):
         imgtags = GExiv2.Metadata(self.source_media.path)
         dest_imgtags = GExiv2.Metadata(self.path)
         for tag in imgtags.get_exif_tags():
-            dest_imgtags[tag] = imgtags[tag]
+            try:
+                dest_imgtags[tag] = imgtags[tag]
+            except UnicodeDecodeError:
+                logging.warning(_("Could not copy metadata tag '%s'"), tag)
 
         new_size = self.get_size()
         dest_imgtags['Exif.Photo.PixelXDimension'] = str(new_size[0])
@@ -211,7 +218,7 @@ class ImageOtherSize(ResizedImage):
     def save(self, im):
         super(ImageOtherSize, self).save(im)
 
-        if self.webgal.config.getboolean('webgal', 'publish-metadata'):
+        if self.webgal.config.get('webgal', 'publish-metadata'):
             self.copy_metadata()
 
 
@@ -225,7 +232,7 @@ class VideoThumb(ResizedImage):
     def get_image(self):
         try:
             thumb = mediautils.VideoThumbnailer(self.source_media.path).get_thumb()
-        except mediautils.TranscodeError as e:
+        except mediautils.VideoError as e:
             logging.error(_("  creating %s thumbnail failed, skipped"),
                           self.source_media.filename)
             logging.info(str(e))
@@ -246,21 +253,18 @@ class WebalbumPicture(make.FileMakeObject):
 
         self.add_dependency(webgal_dir.source_dir)
 
-        # Use already generated thumbs for better performance (lighter to
-        # rotate, etc.).
-        thumbs = [image.thumb
-                  for image in webgal_dir.get_all_medias_tasks()
-                  if image.thumb and not image.media.broken]
+        pics = [p for p in webgal_dir.source_dir.get_all_medias()
+                if not p.broken]
 
-        for thumb in thumbs:
-            self.add_dependency(thumb)
+        for p in pics:
+            self.add_dependency(p)
 
         if webgal_dir.source_dir.album_picture:
             albumpic_path = os.path.join(webgal_dir.source_dir.path,
                                          webgal_dir.source_dir.album_picture)
             if not os.path.isfile(albumpic_path):
                 logging.error(_("Supplied album picture %s does not exist."),
-                              webgal_dir.source_dir.album_picture)
+                              albumpic_path)
 
             md_dirpic_thumb = webgal_dir._add_size_qualifier(
                 webgal_dir.source_dir.album_picture, THUMB_SIZE_NAME)
@@ -268,13 +272,13 @@ class WebalbumPicture(make.FileMakeObject):
         else:
             md_dirpic_thumb = None
 
-        pics = [thumb.path for thumb in thumbs]
+        pic_paths = [p.path for p in pics]
 
         multipic_repr = eyecandy.WEBALBUMPIC_TYPES[webgal_dir.webalbumpic_type]
 
         # Use 800x600 as a random value to obtain a 4:3 aspect ratio (if
         # thumb size preserves aspect ratio)
-        self.dirpic = multipic_repr(pics, md_dirpic_thumb,
+        self.dirpic = multipic_repr(pic_paths, md_dirpic_thumb,
                                     bg=webgal_dir.webalbumpic_bg,
                                     result_size=webgal_dir.webalbumpic_size)
 
@@ -316,7 +320,7 @@ class WebVideo(genfile.WebalbumFile):
                                                    self.new_height)
             transcoder.set_progress(self.progress)
             transcoder.convert(self.path)
-        except mediautils.TranscodeError as e:
+        except mediautils.VideoError as e:
             logging.error(_("  transcoding %s failed, skipped"),
                           self.source_video.filename)
             logging.info(str(e))
