@@ -25,6 +25,9 @@ import multiprocessing
 from . import py2compat
 
 
+class VideoError(Exception): pass
+
+
 try:
     import gi
     try:
@@ -32,12 +35,16 @@ try:
     except ValueError:
         raise ImportError
     from gi.repository import GObject, GLib, Gst
+    if '__getitem__' not in dir(Gst.Caps):
+        logging.warning('Missing `python-gst-1.0` overrides')
+        raise ImportError
 except ImportError:
     HAVE_GST = False
 else:
     HAVE_GST = True
-    gobjects_threads_init = False
+    gst_init = False
     GstPbutils = False
+    gi.require_version('GstPbutils', '1.0')
 
 
 class InterruptHandler(object):
@@ -66,28 +73,24 @@ class InterruptHandler(object):
         return True
 
 
-def gobject_init():
-    global gobjects_threads_init
-    GObject.threads_init()
+def GST_init():
+    global gst_init
     Gst.init(None)
 
     global GstPbutils
     from gi.repository import GstPbutils
 
-    gobjects_threads_init = True
+    gst_init = True
 
 
 from PIL import Image as PILImage
 
 
-class TranscodeError(Exception): pass
-
-
 class GstVideoOpener(object):
 
     def __init__(self, input_file):
-        if not gobjects_threads_init:
-            gobject_init()
+        if not gst_init:
+            GST_init()
 
         self.input_file = py2compat.u(input_file,
                                       sys.getfilesystemencoding())
@@ -143,7 +146,9 @@ class GstVideoOpener(object):
                 self.pipeline.query_duration(Gst.Format.TIME)
             if md_success:
                 self.media_duration = media_duration
-        if self.progress is not None and self.media_duration > 0:
+
+        if self.progress is not None and self.media_duration is not None\
+        and self.media_duration > 0:
             self.progress.set_task_progress(100 * current_position
                                             // self.media_duration)
 
@@ -189,7 +194,7 @@ class GstVideoOpener(object):
                     self.__stop_pipeline()
                 elif message.type == Gst.MessageType.ERROR:
                     self.__stop_pipeline()
-                    raise TranscodeError(message.parse_error())
+                    raise VideoError(message.parse_error())
                 elif message.type == Gst.MessageType.APPLICATION:
                     if message.src == self.pipeline:
                         struct_name = message.get_structure().get_name()
@@ -197,7 +202,7 @@ class GstVideoOpener(object):
                             self.__stop_pipeline()
                         elif struct_name == 'stalled':
                             self.__stop_pipeline()
-                            raise TranscodeError('Pipeline is stalled, this is a problem either in gst or in lazygal\'s use of gst')
+                            raise VideoError('Pipeline is stalled, this is a problem either in gst or in lazygal\'s use of gst')
 
         if self.progress is not None:
             self.progress.set_task_done()
@@ -207,23 +212,25 @@ class GstVideoOpener(object):
             Gst.Structure.new_empty('aborded_playback'))
         self.pipeline.get_bus().post(msg)
 
+        if self.progress is not None:
+            self.progress.set_task_done()
+
 
 class GstVideoInfo(object):
 
     def __init__(self, path):
-        self.path = Gst.filename_to_uri(py2compat.u(path,
-            sys.getfilesystemencoding()))
+        self.path = py2compat.u(path, sys.getfilesystemencoding())
 
     def inspect(self):
         # Init gobjects threads only if an inspection is initiated
-        if not gobjects_threads_init:
-            gobject_init()
+        if not gst_init:
+            GST_init()
 
         discoverer = GstPbutils.Discoverer()
         try:
-            info = discoverer.discover_uri(self.path)
+            info = discoverer.discover_uri(Gst.filename_to_uri(self.path))
         except GObject.GError as e:
-            raise TranscodeError(e)
+            raise VideoError(e)
 
         vinfo_caps = info.get_video_streams()[0].get_caps()[0]
         self.videowidth = vinfo_caps['width']
