@@ -65,6 +65,13 @@ GEXIV2_DATE_FORMAT = '%Y:%m:%d %H:%M:%S'
 GExiv2.log_set_level(GExiv2.LogLevel.MUTE) # hide exiv2 errors
 
 
+TAGS_OF_INTEREST = (
+    'Exif.Photo.DateTimeOriginal',
+    'Exif.Photo.DateTimeDigitized',
+    'Exif.Image.DateTime',
+)
+
+
 def decode_exif_user_comment(raw, imgpath):
     """
     GExiv2 does not decode EXIF user comment.
@@ -144,8 +151,7 @@ class ImageInfoTags(object):
         software when editing photos later.
         """
         for tag in ('Exif.Photo.DateTimeOriginal',
-                    'Exif.Image.DateTimeDigitized',
-                    'Exif.Image.DateTime',
+                    'Exif.Photo.DateTimeDigitized',
                    ):
             try:
                 dt_str = self._metadata[tag]
@@ -249,6 +255,17 @@ class ImageInfoTags(object):
         except UnicodeDecodeError:
             return encoded_string.decode(encoding, 'replace')
 
+    def _tag_fallback_to_encoding(self, tag, encoding=FALLBACK_ENCODING):
+        try:
+            v = self._metadata[tag]
+        except UnicodeDecodeError:
+            try:
+                v = self._metadata.get_raw(tag)
+            except AttributeError: # GExiv2 < 0.10.3
+                logging.warning(_("Encoding for '%s' is bad, ignoring"), tag)
+                v = ''
+        return self._fallback_to_encoding(v, encoding)
+
     def get_exif_usercomment(self):
         ret = self._metadata['Exif.Photo.UserComment'].strip(' \0\x00\n')
         ret = decode_exif_user_comment(ret, self.image_path)
@@ -365,8 +382,7 @@ class ImageInfoTags(object):
 
     def get_authorship(self):
         try:
-            author = self._metadata['Exif.Image.Artist']
-            return self._fallback_to_encoding(author)
+            return self._tag_fallback_to_encoding('Exif.Image.Artist')
         except KeyError:
             return ''
 
@@ -401,6 +417,10 @@ class ImageInfoTags(object):
 
         return kw
 
+    def of_interest(self):
+        return { tag: self._metadata[tag]
+                 for tag in TAGS_OF_INTEREST if tag in self._metadata }
+
 
 class NoMetadata(Exception):
     """
@@ -442,28 +462,27 @@ class DirectoryMetadata(make.GroupTask):
         if path is None or not os.path.exists(path):
             raise NoMetadata(_('Could not open metadata file %s') % path)
 
-        f = open(path, 'r')
-        for line in f:
-            for tag in MATEW_TAGS.keys():
-                tag_text = MATEW_TAGS[tag]
-                tag_len = len(tag_text)
-                if line[:tag_len] == tag_text:
-                    data = line[tag_len:]
-                    data = data.strip()
-                    # Strip quotes
-                    if data[0] == '"':
-                        data = data[1:]
-                    if data[-1] == '"':
-                        data = data[:-1]
-                    data = py2compat.u(data, FILE_METADATA_ENCODING)
+        with open(path, 'rb') as f:
+            for line in f:
+                for tag in MATEW_TAGS.keys():
+                    tag_text = MATEW_TAGS[tag]
+                    tag_len = len(tag_text)
+                    if line[:tag_len] == tag_text:
+                        data = line[tag_len:]
+                        data = data.strip()
+                        # Strip quotes
+                        if data[0] == '"':
+                            data = data[1:]
+                        if data[-1] == '"':
+                            data = data[:-1]
+                        data = data.decode(FILE_METADATA_ENCODING)
 
-                    if tag == 'album_picture':
-                        if subdir is not None:
-                            data = os.path.join(subdir, data)
-                        data = os.path.join(self.dir_path, data)
+                        if tag == 'album_picture':
+                            if subdir is not None:
+                                data = os.path.join(subdir, data)
 
-                    metadata[tag] = data
-                    break
+                        metadata[tag] = data
+                        break
 
         return metadata
 
@@ -516,7 +535,7 @@ class DirectoryMetadata(make.GroupTask):
             except IndexError:
                 picture = None
             if picture is not None:
-                result['album_picture'] = picture
+                result['album_picture'] = os.path.relpath(picture, dir.path)
 
         return result
 
